﻿//Code for the Power Split Device demo
//Author: Alex Hart http://eahart.com

/*

Copyright (c) 2006, Alex Hart
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

    * Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in the
	documentation and/or other materials provided with the distribution.
    * Neither the name of Alex Hart nor the names of its
	contributors may be used to endorse or promote products derived
	from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

*/

class DrawNomograph
{
	private var Point1 : Object = { x:0, y:0 };
	private var Point2 : Object = { x:0, y:0 };
	private var Point3 : Object = { x:0, y:0 };
	
	private var OverlaySafeColor : Number = 0x000000;	
	private var OverlaySafeAlpha : Number = 18;
	private var OverlaySafeLineWt : Number = 0.5;
	private var OverlayWarningColor : Number = 0xFF0000;
	private var OverlayWarningAlpha : Number = 100;
	//private var myMC : MovieClip;
	
	private var mTarget1 : MovieClip;
	private var mTarget2 : MovieClip;
	private var mTarget3 : MovieClip;
	private var mSubTarget : String;
	
	private var m1 : MovieClip;
	private var m2 : MovieClip;
	private var m3 : MovieClip;

	public static var NomographScaleFactor : Number = 1;
	public static var MG1OverRPM : Boolean = false;
	public static var mThis : Object;
	public static var mParent : MovieClip;
	
	public function DrawNomograph ( pTarget1 : MovieClip,
								    pTarget2 : MovieClip,
									pTarget3 : MovieClip,
									pSubTarget : String,
									pParent : MovieClip )
	{
		mTarget1 = pTarget1;
		mTarget2 = pTarget2;
		mTarget3 = pTarget3;
		mSubTarget = pSubTarget;
		mParent = pParent;
		mThis = this;
		//myMC = mParent.createEmptyMovieClip("Loop_mc", 998);
		
		m1 = eval(mTarget1 + "." + mSubTarget);
		m2 = eval(mTarget2 + "." + mSubTarget);
		m3 = eval(mTarget3 + "." + mSubTarget);
	}
	
	private function Activate ()
	{
		updatePoints();
		Draw();
		mParent.line_mc._visible = true;
		//myMC.onEnterFrame = Draw;
	}
	private function Deactivate ()
	{
		mParent.line_mc._visible = false;
		//delete myMC.onEnterFrame;
	}

	/////////
	// Run by myMC. path reference needed
	
	private function Draw ()
	{
		delete myLineMC;
		//mThis.updatePoints();
		var myLineMC : MovieClip;
		myLineMC = mParent.createEmptyMovieClip("line_mc", 333 );
		if ( MG1OverRPM ) {
			myLineMC.lineStyle(2, OverlayWarningColor, OverlayWarningAlpha);
		} else {
			myLineMC.lineStyle(OverlaySafeLineWt, OverlaySafeColor, OverlaySafeAlpha);
		}
		myLineMC.moveTo( Point1.x, Point1.y / NomographScaleFactor );
		myLineMC.lineTo( Point2.x, Point2.y / NomographScaleFactor );
		myLineMC.lineTo( Point3.x, Point3.y / NomographScaleFactor );
	}
	public function updatePoints () : Void
	{
		Point1.x = m1._x;
		Point1.y = m1._y;
		Point2.x = m2._x;
		Point2.y = m2._y;
		Point3.x = m3._x;
		Point3.y = m3._y;
		mTarget1.localToGlobal(Point1);
		mTarget2.localToGlobal(Point2);
		mTarget3.localToGlobal(Point3);
		Draw();
	}
	
	public function setActive ( pBool ) : Void
	{
		if ( pBool )
		{
			Activate();
		} else
		{
			Deactivate();
		}
	}
	public function setLineProperties ( pColor : Number,
									    pAlpha : Number,
										pWeight : Number ) : Void
	{
		OverlaySafeColor = pColor;	
		OverlaySafeAlpha = pAlpha;
		OverlaySafeLineWt = pWeight;
	}
	public function setWarning ( pBool : Boolean ) : Void
	{
		MG1OverRPM = pBool;
	}
	public static function getWarning () : Boolean
	{
		return MG1OverRPM;
	}
	public static function setScaleFactor ( pScaleFactor : Number ) : Void
	{
		NomographScaleFactor = pScaleFactor;
	}
	public static function getScaleFactor () : Number
	{
		return NomographScaleFactor;
	}
}
