% ************************************************************************
% This script initiates all parameters, look up tables etc needed for
% simulation of a "parallel" hybrid Simulink model.
%
% The script is subject to continuous development and changes and a good
% understaning of the relation the Simulink program is needed for a useful
% outcome.
%
% Mats Alakla, Fot, January 24, 2016.
%
% ************************************************************************
clear all

rho_air = 1.2;  % Air density
grav = 9.81;    % Gravitational constant

Wbatt=10000*3.6e6;
Mv = 36287;         % Vehicle curb weight 80000 pounds
rw = 0.506;	        % wheel radius (m)
Cd = 0.36; 	        % air_resistance. (Sports Car 0.3-0.4, Ecomony Car 0.4-0.5, Pickup Truck 0.5, Tractor-Trailer,with fairings 0.6-0.7, Tractor-Trailer 0.7-0.9)
Cr = 0.0032;	    % roll resistance (0.006...0.008 for low roll resist tires. 0.01...0.015 ordinary tires)
Av = 2.550*3.8;     % Front area as Volvo FE6x2
vmax = 110/3.6;       % max maxhastighet

% ******************** Road and Performance *******************************
Max_start_slope = 5; % [%]
% Max_curb_hight = 0.065;  %155;  % [m]
% hc=Max_curb_hight;  
% Maxle = 0.7*Mv;
% Tw_curb = rw*Maxle*grav/sin(acos(-(hc-rw)/rw));

Tw_slope = rw*Mv*grav*sin(atan(Max_start_slope/100));
Tw_roll = Mv*grav*Cr*rw*cos(atan(Max_start_slope/100));
Tw_drag = rw*0.5*Av*Cd*rho_air*vmax^2;
%Tw_acc = Maxle*(50/3.6/17.4)*rw;

v=[1:vmax*3.6]/3.6;  % [m/s]

Tmax = Tw_slope + Tw_roll + rw*0.5*Av*Cd*rho_air*v.^2; % Needed to drive and accelerate
Pmax=Tmax.*v/rw;

plot(v*3.6,Pmax/1000)
grid
hold on
xlabel('Speed [km/h]')
ylabel('Wheel Power [kW]')


Powers=[100:100:800];


