% ************************************************************************
% This script initiates all parameters, look up tables etc needed for
% simulation of a "parallel" hybrid Simulink model.
%
% The script is subject to continuous development and changes and a good
% understaning of the relation the Simulink program is needed for a useful
% outcome.
%
% Mats Alakla, Fot, January 24, 2016.
%
% ************************************************************************

rho_air = 1.2;  % Air density
grav = 9.81;    % Gravitational constant

% *** Select Drive Cycle, Vehicle type, Fuel=Engine and Battery Size
Drive_cycle = 8;
Vehicle = 4;
Fuel = 2;

% *** Load drive cycles **************************************************
% The following "switch" selects a drive cycle where the first 12
% alternatives are according to an "Lund University standard" and cycle
% 13ff are according to the Volvo VSIM standard. IN any case, the data is
% boiled down to a matrix "cycle" with 6 columns: 1) time [s], 2) speed
% reference [m/s], 3) position [m], 4) altitude [m], 5) slope [rad] and 6)
% slide in availability [-]. The "slide in availability" is in some models
% used to indicathe the presence of an Electric Road, but that is not used
% in the Parallel Hybrid model supplied with this script.

clear cycle
switch Drive_cycle
    case 1 % nedc
        load ..\DriveCycles\nedc;
        cycle = nedc;
    case 2 % us06;
        load ..\DriveCycles\us06;
        cycle = us06;
    case 3 % Bus 1
        load ..\DriveCycles\london159;
        cycle = london159;
    case 4 % Bus 2
        load ..\DriveCycles\cbr85;
        cycle = cbr85;
    case 5 % Distribution 1
        load ..\DriveCycles\Distr1;
        cycle = Distr1;
    case 6 % Long Haul 1
        load ..\DriveCycles\LongHaul1;
        cycle = LongHaul1;
    case 7 % Long Haul 2
        load ..\DriveCycles\LongHaul2;
        cycle = LongHaul2;
    case 8 % Long Haul 3
        load ..\DriveCycles\LongHaul3;
        cycle = LongHaul3;
    case 9 % Long Haul 4
        load ..\DriveCycles\LongHaul4;
        cycle = LongHaul4;
    case 10 % ramp
        load ..\DriveCycles\Ramp;
        cycle = Ramp;
    case 11 % Landskrona Slide IN Bus
        load ..\DriveCycles\LandskronaSI
        cycle = LAcycle;
    case 12 % SwedenRoute
        load ..\DriveCycles\SwedenRoute
        cycle = SwedenRoute;       
end

if Drive_cycle>12 & Drive_cycle<17
    dalt = [0; slask.dist.alt(2:length(slask.dist.alt))-slask.dist.alt(1:length(slask.dist.alt)-1)];
    ddist = [eps; slask.dist.pos(2:length(slask.dist.pos))-slask.dist.pos(1:length(slask.dist.pos)-1)];
    slask.dist.slope = atan(dalt./ddist);
    clear cycle
    cycle = [slask.time.t_ref slask.time.v_ref slask.dist.pos slask.dist.alt slask.dist.slope zeros(length(slask.time.t_ref),1)];
end

endtime = max(cycle(:,1));

cycle(1:length(cycle),6)=0; % make sure that no "Slide In" is indicated

% *** Vehicle Choise *****************************************************
% In this section the main vehicle parameters are selected, for 4 different
% types of vehicles.

switch Vehicle
    case 1 % Car
        Mv = 1600; % Vehicle weight
        rw = 0.30;	        % wheel radius (m)
        Cd = 0.26; 	        % air_resistance
        Cr = 0.008;	        % roll resistance
        Av = 2.55;	        % Front area
        vmax=172/3.6;       % 172 km/h maxhastighet
        Pmax=100000;
        Number_of_gears = 6;
        MinShiftTime = 0.5;
        Paux = 1000;         % Without AC
    case 2 % City Bus (Coman Maxirider)
        Wbatt=78*3.6e6;
        Mv = 5990;       % Vehicle curb weight 
        rw = 0.349;   %0.506;	        % wheel radius (m)
        Cd = 0.79; 	        % air_resistance. (Sports Car 0.3-0.4, Ecomony Car 0.4-0.5, Pickup Truck 0.5, Tractor-Trailer,with fairings 0.6-0.7, Tractor-Trailer 0.7-0.9)
        Cr = 0.00576;	    % roll resistance (0.006...0.008 for low roll resist tires. 0.01...0.015 ordinary tires)
        Av = 2.23*2.7;   % Front area as Volvo FE6x2
        vmax = 90/3.6;       % max maxhastighet
        Pem_max = 120000;
        Number_of_gears = 2;
        MinShiftTime = 2;
        % Paux = 2000;         % Without AC
    case 3 %  City Bus (Volvo 7900 Electric) 
        Wbatt=76*3.6e6;
        Mv = 19000;         % Vehicle curb weight + 250 kg passenger and cargo
        rw = 0.506;	        % wheel radius (m)
        Cd = 0.79; 	        % air_resistance. (Sports Car 0.3-0.4, Ecomony Car 0.4-0.5, Pickup Truck 0.5, Tractor-Trailer,with fairings 0.6-0.7, Tractor-Trailer 0.7-0.9)
        Cr = 0.00576;	    % roll resistance (0.006...0.008 for low roll resist tires. 0.01...0.015 ordinary tires)
        Av = 2.55*3.28;   % Front area as Volvo FE6x2
        vmax=90/3.6;       % max maxhastighet
        Pem_max = 160000;
        Number_of_gears = 2;
        MinShiftTime = 2;
        % Paux = 4000;      % Without AC
    case 4 % Long Haul Truck
        Wbatt=10000*3.6e6;
        Mv = 40000;         % Vehicle curb weight + 250 kg passenger and cargo
        rw = 0.506;	        % wheel radius (m)
        Cd = 0.3; 	        % air_resistance. (Sports Car 0.3-0.4, Ecomony Car 0.4-0.5, Pickup Truck 0.5, Tractor-Trailer,with fairings 0.6-0.7, Tractor-Trailer 0.7-0.9)
        Cr = 0.0032;	    % roll resistance (0.006...0.008 for low roll resist tires. 0.01...0.015 ordinary tires)
        Av = 2.550*3.8;     % Front area as Volvo FE6x2
        vmax = 90/3.6;       % max maxhastighet
        Pem_max = 450000;
        Number_of_gears = 12;
        MinShiftTime = 1;
        Paux = 3000;    % Without AC
end

% ******************** Road and Performance *******************************
Max_start_slope = 8; % [%]
Max_curb_hight = 0.065;  %155;  % [m]
hc=Max_curb_hight;  
Maxle = 2100;  %0.7*Mv;
Tw_curb = rw*Maxle*grav/sin(acos(-(hc-rw)/rw));
Tw_slope = rw*Mv*grav*sin(atan(Max_start_slope/100));
Tw_roll = Mv*grav*Cr*rw;
Tw_drag = rw*0.5*Av*Cd*rho_air*vmax^2;
Tw_acc = Maxle*(50/3.6/17.4)*rw;

v=[0:vmax*3.6]/3.6;  % [m/s]

v_base_set=0;
for i=1:length(v)
    if i==1
        Tmax(i)=Tw_curb+Tw_slope; % Needed to start
    else
        Tmax(i) = min(Tw_acc + Tw_slope + Tw_roll + rw*0.5*Av*Cd*rho_air*v(i)^2,Pem_max/(v(i)/rw)); % Needed to drive and accelerate
        Tdrag(i) = rw*0.5*Av*Cd*rho_air*v(i)^2;
    end
    if (i>3 & Tmax(i)<Tmax(i-1) & v_base_set==0);
        v_base = v(i);
        v_base_set = 1;
    end
end

% *** Electric machine parameters  ***************************************
% The following lines set the EM drive performance

fwr = 3; % Field weakening ratio, i.e. ratio between max speed and base
%  speed
wem_min = 0;
wem_max = 7700*pi/30; % EM max speed
wem_vmax = 7700*pi/30; % EM Speed at max vehicle speed
Tem_max = Pem_max/(wem_max/fwr);  % Peak continuous torque
% The call for "CreateEMmap" is used for the EM in the same way as the call
% for i "CreateICEmap_special" s used regarding the ICE. See related
% comment for the ICE above. 
[PtoTem,EtaEM,Tem,Wem,Tem_lim,EnergyConsEM] = CreateEMmap(Pem_max,wem_max,Tem_max);

% *** Transmission settings **********************************************

utvx_min = wem_vmax/(1.5*vmax/rw);  % "Highest gear". 12.3;  
% utvx_max = wice_max/((vmax/7.5)/rw); % "Gear one".
utvx_max = Tmax(1)/max(Tem_lim); % "Gear one".

DCT=1; % parameter for the transmission to eleminate torque interruptions 
% when gearshifting

% the following lines create a sequence of gear ratios
Utvx_vect = zeros(1,Number_of_gears+1);
Utvx_vect(1,1) = utvx_min;
Utvx_vect(1,length(Utvx_vect(1,:))) = inf;
for i=2:Number_of_gears,
    Utvx_vect(1,i) =  Utvx_vect(1,i-1)*(utvx_max/utvx_min)^(1/(Number_of_gears-1));
end

EtaGEAR = 0.97; % Simplest possible efficiency model of the transmission

gr1 = utvx_max;
gr2 = utvx_min;

figure(20),clf
plot(v*3.6,Tdrag,'b--'); text(85,max(Tw_drag),'Tw drag')
hold on
plot(v*3.6,Tw_roll*ones(size(Tmax)),'r--'); text(5,max(Tw_roll),'Tw roll')
plot(v*3.6,Tw_slope*ones(size(Tmax)),'g--'); text(5,max(Tw_slope),'Tw slope')
plot(v*3.6,Tw_curb*ones(size(Tmax)),'m--'); text(5,max(Tw_curb),'Tw curb')
plot(v*3.6,Tw_acc*ones(size(Tmax)),'c--'); text(5,max(Tw_acc),'Tw acc')
plot(v*3.6,Tmax,'*'); text(v_base*3.6,1.1*max(Tmax(1,2:91)),[num2str(Pem_max/1000) ' [kW]'])
grid

plot(Wem*rw*3.6/gr1,Tem_lim*gr1,'r'); text(40,Tmax(1)/2*1.05,['Ettan = ' num2str(floor(10*gr1)/10)])
plot(Wem*rw*3.6/gr2,Tem_lim*gr2,'g'); text(40,Tmax(1)/2*0.95,['Tvan = ' num2str(floor(10*gr2)/10)])

xlabel('Speed [m/s]')
ylabel('Wheel torque [Nm]')
legend('Drag Torque','Roll Torque','Slope torque (8%)',['Curb Torque ' num2str(Max_curb_hight) ' [m]'],'Accel Torque ("0..50 in 17 sec")','Max Wheel Torque')


% *** Power Electronics efficiency ***************************************
% Since the simulation program is design NOT to express Voltages and
% Currents, only Powers, the simplest form of PEC efficiency mode is used.

EtaPE = 0.97; % PEC efficiency

% *** Battery parameters *************************************************
% The battery model is a constant voltage in series with a resistance. The
% electric variables voltage and current is recalculated into termina power
% and efficiencies. this is done in the call to "CreateBATTmap".

[EtaBATT,Pbatt,Mbatt,Pbatt_max_new,Wbatt_new]=CreateBATTmap(Pem_max,Wbatt);
SOC_batt_ref_value = 20; % target value for SOC control
SOC_batt_start_value = 90; % Start value of SOC
SOC_tract_max = 95; % Max SOC level, no negative EM power allowed above.
SOC_tract_min = 15; % Min SOC level, no positive EM power allowed below.

% *** Controller parameters **********************************************

Tau_charge = 3;   % Dynamic limitation for the ICE operation

ksoc = max(Wbatt)/400/Tau_charge; % Max SOC control gain with respect to
% symmetric optimum  
ksoc_vect = ksoc*tan(pi/2.05*[-1:1/15:1])*2; % Gain vector for SOC control



