﻿//Code for the Power Split Device demo
//Author: Alex Hart http://eahart.com

/*

Copyright (c) 2006, Alex Hart
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

    * Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in the
	documentation and/or other materials provided with the distribution.
    * Neither the name of Alex Hart nor the names of its
	contributors may be used to endorse or promote products derived
	from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

*/
class EngineAttrs
{
	private var mRunTimer : Number = getTimer();
	private var mElapsedTime : Number = 0;

	// Scale for the speed of the animation. Inform the viewer of the scale.
	private static var mRotationScale : Number = 400;

	//Some constants. Based on other people's knowledge of 2nd Gen Prius geometry, so may be subject to error/change
	
	private static var mMG2toMPHConversionConstant : Number = 59.14;
	
	private static var mMaxReverseMPH : Number = -25;
	
	private static var mICEMinRPM : Number = 1000;
	private static var mICEMaxRPM : Number = 4500;
	private static var mMG2MaxRPM : Number = 6500;
	private static var mMG2MinRPM : Number = MaxReverseMPH * MG2toMPHConversionConstant;
	private static var mMG1MaxRPM : Number = 6500;
	private static var mMG1MinRPM : Number = -6500;
	
	//Declare some variables
	
	private static var mRingRotation : Number = 0;
	private static var mPlanetCarrierRotation : Number = 0;
	private static var mRingRotationDelta : Number = 0;
	private static var mPlanetCarrierRotationDelta : Number = 0;
	private static var mSunRotation : Number = 0;
	
	private static var mMG2RPM : Number = 0;
	private static var mICERPM : Number = 0;
	private static var mMG1RPM : Number = 0;
	private static var mMPH : Number = 0;
	private static var mMPG : Number = 0;
	private static var mSOC : Number = 0;
	
	public static var mThis : Object;
	public static var mParent : MovieClip;
	
	public function EngineAttrs ( pParent : MovieClip )
	{
		mParent = pParent;
		//var myMC : MovieClip = mParent.createEmptyMovieClip("EngineLoop_mc", 1002);
		//myMC.onEnterFrame = updateEngine;
		mParent.ScaleDisplay_txt.text = "Gear rotation is 1/" + mRotationScale + "th of actual speed."
	}
	
	//every frame:
	public function updateEngine () : Void
	{
		//Number of seconds since the last frame was displayed (getTimer returns milliseconds):
		mElapsedTime = ( getTimer() - mRunTimer ) / 1000;
		mRunTimer = getTimer();

		//Fake MPG and SOC if CAN-view is not connected
		if ( !mParent.myCarConnection.getCanViewStatus() ) {
			mMPG = (Math.sin( getTimer()/2000 ) + 1)*127.5; //fake the MPG number
			mSOC = ((Math.sin( getTimer()/1600 ) + 1)*20 ) + 40; //fake the SOC number
		}
		mMPH = Math.round( mMG2RPM / mMG2toMPHConversionConstant );
		updateMG1();
		
		// Change in rotation since the last frame was displayed.
		// Current RPM divided by 60 to get Revolutions Per Second
		// Multiply by elapsed number of seconds to get how many revolutions in the elapsed time.
		// Multiply by 360 to get number of degrees delta since last update.
	
		mRingRotationDelta = mElapsedTime * ( mMG2RPM / 60 ) * 360;
		mPlanetCarrierRotationDelta = mElapsedTime * ( mICERPM / 60 ) * 360;
		
		// Add the scaled rotation to the rotation variables.
		mRingRotation += mRingRotationDelta / mRotationScale;
		mPlanetCarrierRotation += mPlanetCarrierRotationDelta / mRotationScale;
	
		mSunRotation = (( 3.6 * mPlanetCarrierRotation ) - ( 2.6 * mRingRotation )) % 360;
		//for (var i in EngineAttrs){
			//trace(i + ": " + EngineAttrs[i]);
		//}
	}
	public function updateMG1 () : Number
	{
		mMG1RPM = ( 3.6 * mICERPM ) - ( 2.6 * mMG2RPM );
		return mMG1RPM;
	}
	
	//getters and settters

	/**********/
	public static function  get RotationScale() : Number
	{
		return mRotationScale;
	}
	public static function  set RotationScale( p : Number ) : Void
	{
		mRotationScale = p;
	}
	/**********/
	public static function get MG2toMPHConversionConstant() : Number
	{
		return mMG2toMPHConversionConstant;
	}
	public static function set MG2toMPHConversionConstant( p : Number ) : Void
	{
		mMG2toMPHConversionConstant = p;
	}
	/**********/
	public static function get MaxReverseMPH() : Number
	{
		return mMaxReverseMPH;
	}
	public static function set MaxReverseMPH( p : Number ) : Void
	{
		mMaxReverseMPH = p;
	}
	/**********/
	public static function get ICEMinRPM() : Number
	{
		return mICEMinRPM;
	}
	public static function set ICEMinRPM( p : Number ) : Void
	{
		mICEMinRPM = p;
	}
	/**********/
	public static function get ICEMaxRPM() : Number
	{
		return mICEMaxRPM;
	}
	public static function set ICEMaxRPM( p : Number ) : Void
	{
		mICEMaxRPM = p;
	}
	/**********/
	public static function get MG2MaxRPM() : Number
	{
		return mMG2MaxRPM;
	}
	public static function set MG2MaxRPM( p : Number ) : Void
	{
		mMG2MaxRPM = p;
	}
	/**********/
	public static function get MG2MinRPM() : Number
	{
		return mMG2MinRPM;
	}
	public static function set MG2MinRPM( p : Number ) : Void
	{
		mMG2MinRPM = p;
	}
	/**********/
	public static function get MG1MaxRPM() : Number
	{
		return mMG1MaxRPM;
	}
	public static function set MG1MaxRPM( p : Number ) : Void
	{
		mMG1MaxRPM = p;
	}
	/**********/
	public static function get MG1MinRPM() : Number
	{
		return mMG1MinRPM;
	}
	public static function set MG1MinRPM( p : Number ) : Void
	{
		mMG1MinRPM = p;
	}
	/**********/
	public static function get RingRotation() : Number
	{
		return mRingRotation;
	}
	public static function set RingRotation( p : Number ) : Void
	{
		mRingRotation = p;
	}
	/**********/
	public static function get PlanetCarrierRotation() : Number
	{
		return mPlanetCarrierRotation;
	}
	public static function set PlanetCarrierRotation( p : Number ) : Void
	{
		mPlanetCarrierRotation = p;
	}
	/**********/
	public static function get RingRotationDelta() : Number
	{
		return mRingRotationDelta;
	}
	public static function set RingRotationDelta( p : Number ) : Void
	{
		mRingRotationDelta = p;
	}
	/**********/
	public static function get PlanetCarrierRotationDelta() : Number
	{
		return mPlanetCarrierRotationDelta;
	}
	public static function set PlanetCarrierRotationDelta( p : Number ) : Void
	{
		mPlanetCarrierRotationDelta = p;
	}
	/**********/
	public static function get SunRotation() : Number
	{
		return mSunRotation;
	}
	public static function set SunRotation( p : Number ) : Void
	{
		mSunRotation = p;
	}
	/**********/
	public static function get MG2RPM() : Number
	{
		return mMG2RPM;
	}
	public static function set MG2RPM( p : Number ) : Void
	{
		mMG2RPM = p;
	}
	/**********/
	public static function get ICERPM() : Number
	{
		return mICERPM;
	}
	public static function set ICERPM( p : Number ) : Void
	{
		mICERPM = p;
	}
	/**********/
	public static function get MG1RPM() : Number
	{
		return mMG1RPM;
	}
	public static function set MG1RPM( p : Number ) : Void
	{
		mMG1RPM = p;
	}
	/**********/
	public static function get MPH() : Number
	{
		return mMPH;
	}
	public static function set MPH( p : Number ) : Void
	{
		mMPH = p;
	}
	/**********/
	public static function get MPG() : Number
	{
		return mMPG;
	}
	public static function set MPG( p : Number ) : Void
	{
		mMPG = p;
	}
	/**********/
	public static function get SOC() : Number
	{
		return mSOC;
	}
	public static function set SOC( p : Number ) : Void
	{
		mSOC = p;
	}
	/**********/

	
}